source("0_functions.R")

# Reading data ----
int           <- read_delim("data/Interlocks.csv", delim = ";", trim_ws = TRUE)
corp          <- read_delim("data/Active Corporations.csv", delim = ";", trim_ws = TRUE)
affil         <- read_delim("data/Affilation edgelist.csv", delim = ";", trim_ws = TRUE)
active.corp   <- read_delim(file = "data/Active Corporations.csv", delim = ";", trim_ws = TRUE)
bank          <- read_delim("data/chem dir on bank boards.csv", delim = ";")

period        <- 1950:2015
period.names  <- as.character(period)

l.act           <- apply(active.corp, 1, function(x) x["Start"]:x["End"])
names(l.act)    <- active.corp$Name
number.of.nodes <- stack(table(stack(l.act)$values))$values

# Interlock approach -----
intB                      <- int[, c(2 ,1, 3:6, 8, 7, 9:ncol(int))]
colnames(intB)[c(1:2, 7:8)]       <- colnames(int)[c(1:2, 7:8)] 

intC                      <- rbind(int, intB)

intC$"Director A is MB"    <- intC$`Type of Director Comp A` %in% c("CEO/Chair MB", "MB member")
intC$"Director B is MB"    <- intC$`Type of Director Comp B` %in% c("CEO/Chair MB", "MB member")

int.set       <- intC[intC$`Company B` != "VCI" & intC$`Company A` != "VCI",]

graph.all          <- graph_from_data_frame(int.set, directed = TRUE, vertices = corp)

g.list             <- lapply(period, slice_to_year, g = graph.all, delete.vertices = TRUE)
names(g.list)      <- period
g.list.intC        <- g.list

g.list.simple      <- lapply(g.list, igraph::simplify) 
g.list.undirected  <- lapply(g.list.simple, igraph::as.undirected)
g.list.reach       <- lapply(g.list.undirected, connect.neighborhood, order = 2)

# Slices ----

bank     <- bank[, c(1:4, 7:8)]
colnames(bank) <- colnames(intC[, c(1:4, 7:8)])
intCbank  <- rbind(intC[, c(1:4, 7:8)], bank)

intCbank$`Type of Director Comp B` <- fct_recode(intCbank$`Type of Director Comp B`, "CEO" = "CEO/Chair MB" , "Chairman" = "Chairman SB", "Other" = "Employee Rep SB",
                                                 "Other" = "Honorary chairman SB", "Executive" = "MB member" , "Other" = "SB member" , "Other" = "VCI vorstand" )

intCbank$`Type of Director Comp A` <- fct_recode(intCbank$`Type of Director Comp A`, "CEO" = "CEO/Chair MB" , "Chairman" = "Chairman SB", "Other" = "Employee Rep SB",
                                                 "Other" = "Honorary chairman SB", "Executive" = "MB member" , "Other" = "SB member" , "Other" = "VCI vorstand" )


graph.all.bank      <- graph_from_data_frame(intCbank, directed = TRUE)
slice.all           <- lapply(period, slice_to_year, g = graph.all.bank, delete.vertices = FALSE)
names(slice.all)    <- period
slices <- slice.all

slices                <- add.measure.to.slices(slices, label = "degree.current.year", degree)

graph.no.bank         <- delete.vertices(graph.all.bank, v = which(V(graph.all.bank)$name %in% c("VCI", "Deutsche Bank AB",
                                                                                       "Commerzbank SB", "Deutsche Bank SB",
                                                                                       "Dresdner Bank AB", "Dresdner Bank SB", "Commerzbank AB",
                                                                                       "RMB (Dresdner Bank) SB", "SDB (Deutsche Bank) SB", "CDB (Commerzbank) SB",
                                                                                       "RRB (Dresdner Bank) SB", "HKB (Dresdner Bank) SB", "RWB (Deutsche Bank) SB",
                                                                                       "BWD (Commerzbank) SB", "(Dresdner Bank) SB", "(Deutsche Bank) SB")))


slices.no.bank      <- lapply(period, slice_to_year, g = graph.no.bank, delete.vertices = FALSE)
names(slices.no.bank) <- period

slices.no.bank      <- add.measure.to.slices(slices.no.bank, label = "degree.current.year", degree)



# Affiliation edgelist approach ----
# Also called DEN format - see the eliter - package from github.com/antongrau/eliter

den               <- select(affil, NAME = `Full name`, AFFILIATION = Company, PERSON_START = "Begin year", PERSON_END = "End year", ROLE = Type)
den               <- den[-which(is.na(den$NAME)),]
den$PERSON_END    <- as.Date(ISOdate(den$PERSON_END, 12, 1))
den$PERSON_START  <- as.Date(ISOdate(den$PERSON_START, 12, 1))
den               <- as.den(den)
g                 <- graph_from_data_frame(den)
V(g)$type         <- V(g)$name %in% den$AFFILIATION


# Spell.graph -----
date.period   <- as.Date(ISOdate(period, 1, 1))
spell.graph   <- graph.from.spells(den)
a.list        <- accumulative.adjacency.list(spell.graph, steps = 12)
period.names  <- as.character(year(spell.graph$reference.month + months(as.numeric(names(a.list)))))
names(a.list) <- period.names

incidence.l <- llply(names(a.list), incidence.with.former.collegaues, a.list = a.list, den = den, .progress = "text")
names(incidence.l) <- names(a.list)

to.adj      <- function(x) t(x) %*% x

adj.all     <- lapply(incidence.l, to.adj)

incidence.present.l <- lapply(names(a.list), incidence.active, den = den)
names(incidence.present.l) <- names(a.list)
adj.present  <- lapply(incidence.present.l, to.adj)

incidence.past.l <- list()
for(i in 1:length(incidence.l)) incidence.past.l[[i]] <- (incidence.l[[i]] > 0) - (incidence.present.l[[i]] > 0) 
names(incidence.past.l) <- names(a.list)

adj.past <- list()
for(i in 1:length(adj.all)) adj.past[[i]] <- (to.adj(incidence.l[[i]]) > 0) - (to.adj(incidence.present.l[[i]]) > 0)
names(adj.past) <- names(a.list)

graph.present       <- lapply(adj.present, graph_from_adjacency_matrix, mode = "undirected")
graph.present       <- lapply(graph.present, simplify)
graph.present.reach <- lapply(graph.present, connect.neighborhood, order = 2)
graph.past          <- lapply(adj.past, graph_from_adjacency_matrix, mode = "undirected")

g.list               <- lapply(names(a.list),  past.and.present.graph, graph.present = graph.present, graph.present.reach = graph.present.reach, graph.past = graph.past)
g.period             <- g.list[[1]]
for(i in 2:length(g.list)) g.period <- g.period + g.list[[i]]

graph.past.and.present <- g.list

# Banks -----
colnames(bank)[3:4] <- c("Start", "End")
set.of.banks <- unique(bank$"Company B")

int.bank               <- bind_rows(bank, int.set)
graph.bank             <- graph_from_data_frame(int.bank, directed = FALSE)
g.list.bank            <- lapply(period, slice_to_year, g = graph.bank, delete.vertices = FALSE)
names(g.list.bank)      <- period

# VCI ----
graph.vci      <- graph_from_data_frame(intC, directed = FALSE, vertices = corp)
g.list.vci     <- lapply(period, slice_to_year, g = graph.vci, delete.vertices = TRUE)
names(g.list.vci)      <- period
g.list.vci     <- lapply(g.list.vci, igraph::simplify)


save.image(file = "saved/1_data.Rda")