# Analysis ----
source("0_functions.R")
load("saved/1_data.Rda")

# Analysis interlocks ----
antal.edges        <- sapply(g.list.intC, ecount)
antal.edges.simple <- sapply(g.list.simple, ecount)
antal.edges.undirected <- sapply(g.list.undirected, ecount)
antal.strong.edges <- sapply(g.list.intC, function(x) sum(is.multiple(x)))

antal.nodes        <- sapply(g.list.intC, vcount)

density            <- sapply(g.list.intC, graph.density)
average.degree     <- sapply(g.list.intC, function(x) mean(degree(x, mode = "out")))
average.degree.simple <- sapply(g.list.simple, function(x) mean(degree(x, mode = "out")))

sends.mbs          <- sapply(g.list.intC, sending.MB)
number.mbs.send    <- sapply(sends.mbs, sum)
sends.union        <- sapply(g.list.intC, sending.union)
number.of.union.send <- sapply(sends.union, sum)/2
sends.bank         <- sapply(g.list.intC, sending.bank)
number.of.bank.send <- sapply(sends.bank, sum)/2

andel.af.simple.directed.edges.der.er.MB <- number.mbs.send / antal.edges.simple
andel.af.simple.directed.edges.der.er.union  <- number.of.union.send / antal.edges.undirected
andel.af.simple.directed.edges.der.er.bank  <- number.of.bank.send / antal.edges.undirected

average.mbs.send   <- sapply(sends.mbs, sum) / antal.nodes

list.coms <- lapply(g.list.intC, largest.component)
com.sizes <- sapply(list.coms, vcount)

com.size.uden.bank  <- sapply(g.list.intC, com.no.bank)
com.size.uden.union <- sapply(g.list.intC, com.no.union)

ud.data  <- data.frame(
  "Årstal"      = period,
  "Antal edges" = antal.edges,
  "Antal multiple edges" = antal.strong.edges,
  "Antal noder" = antal.nodes,
  "Density"     = density,
  "Average degree" = average.degree,
  "Average degree simple" = average.degree.simple,
  "Sender MB's" = number.mbs.send,
  "Average MB's send" = average.mbs.send)


# Analysis - Affilation edgelist ----

time.nodes         <- data.frame("Nodes" = sapply(lapply(graph.present, igraph::simplify), vcount),
                                 "Nodes past" = sapply(graph.past.and.present, vcount))
time.edges         <- data.frame("Edges" = sapply(lapply(graph.present, igraph::simplify), ecount),
                                 "Reach" = sapply(lapply(graph.present.reach, igraph::simplify), ecount), check.names = FALSE)
time.edges$"Reach with past"   <- time.edges$"Reach" + sapply(lapply(graph.past, igraph::simplify), ecount) 

t.time.av.reach.share      <- ((time.edges*2) / number.of.nodes) / number.of.nodes
time.average.degree        <- ((time.edges*2) / number.of.nodes)

time.component <- data.frame("Component" = sapply(graph.present, function(x) vcount(largest.component(x))),
                             "Component past" = sapply(graph.past.and.present, function(x) vcount(largest.component(x))),
                             check.names = FALSE)
time.component      <- time.component / number.of.nodes
time.component$year <- period

time.density     <- graph.present %>% map(simplify) %>% map_dbl(graph.density) %>% enframe(name = "year") %>% mutate(year = as.numeric(year))

# VCI and banks ----

VCI.connected  <- lapply(g.list.vci, connected.by.set, set = "VCI")
VCI.connected   <- do.call("rbind", args = VCI.connected)
VCI.connected   <- data.frame(VCI.connected)
VCI.connected$year <- period.names

bank.connected   <- lapply(g.list.bank, connected.by.set, set = set.of.banks)
bank.connected   <- do.call("rbind", args = bank.connected)
bank.connected   <- data.frame(bank.connected)
bank.connected$year <- period.names

t.vci.bank.members <- bind_rows(list("VCI" = VCI.connected, "Banks" = bank.connected), .id = "Type")


save.image(file = "saved/2_analysis.Rda")
