# Plots and figures ----
source("0_functions.R")

load(file = "saved/2_analysis.Rda")

# Figure 1A - Number of active ties ----
t.time.edges <- data.frame(year = period, "Edges" = time.edges$Edges)
md    <- melt(t.time.edges, id.vars = "year")

p     <- ggplot(md, aes(x = year, y = value, group = variable)) + geom_point() + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "") + geom_rangeframe()+ xlab("") + ylab("Number of active edges") + scale_shape_discrete(name = "")
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5)) + scale_y_continuous(limits = c(0,NA), breaks = seq(0,100, by = 5))
p.1A.edges <- p

# Figure 1B - Number of active companies ----
t.time.nodes <- data.frame(year = period, "Nodes" = number.of.nodes)
md    <- melt(t.time.nodes, id.vars = "year")

p     <- ggplot(md, aes(x = year, y = value, group = variable)) + geom_point() + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "") + geom_rangeframe()+ xlab("") + ylab("Number of active nodes") + scale_shape_discrete(name = "")
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5)) + scale_y_continuous(limits = c(0,NA), breaks = seq(0,100, by = 5))
p.1B.nodes <- p

# Figure 1C - Share of companies in the largest component ----
md    <- melt(time.component, id.vars = "year")

p     <- ggplot(md, aes(x = year, y = value, group = variable, shape = variable, color = variable)) + geom_point(size = 1.5) + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "") + geom_rangeframe()+ xlab("") + ylab("Share %") + scale_shape_discrete(name = "") 
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5))
p     <- p + scale_y_continuous(limits = c(0,1), labels = percent) 
p.1C.component <- p

# Figure 1D - Average % of companies in reach ----
md    <- melt(t.time.av.reach.share[, -1] %>% mutate(year = period), id.vars = "year")
p     <- ggplot(md, aes(x = year, y = value, group = variable, shape = variable, color = variable)) + geom_point(size = 1.5) + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "") + geom_rangeframe()+ xlab("") + ylab("Average % within reach") + scale_shape_discrete(name = "")
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5)) + scale_y_continuous(labels = percent)
p.1D.share.reach  <- p

# Figure 1E - Average degree -----
md    <- tibble(year = period, value = time.average.degree$Edges)
p     <- ggplot(md, aes(x = year, y = value)) + geom_point(size = 1.5) + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "") + geom_rangeframe()+ xlab("") + ylab("Average degree - present ties") + scale_shape_discrete(name = "")
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5)) + scale_y_continuous(limits = c(0, NA))
p.1E.average.degree <- p

# Figure 1F - Density ----
p     <- ggplot(time.density, aes(x = year, y = value)) + geom_point(size = 1.5) + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "") + geom_rangeframe()+ xlab("") + ylab("Density - present ties") + scale_shape_discrete(name = "")
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5)) + scale_y_continuous(limits = c(0, NA))
p.1F.density <- p


# Figure 3 - Number of years with direct ties between companies ----
e.list      <- lapply(g.list.simple, get.edgelist)
all.edges   <- do.call(rbind, e.list)
graph.all   <- graph_from_edgelist(all.edges)
E(graph.all)$weight <- 1
graph.heat   <- igraph::simplify(graph.all)
t.heat       <- graph.heat[]

p.2.heat     <- heatmap(graph.heat)

# Figure 3A - Share of primary ties ----
d.ties <- data.frame(Year = period.names, "Primary ties" = andel.af.simple.directed.edges.der.er.MB, check.names = FALSE)
t.primary.ties <- d.ties
md    <- melt(d.ties, id.vars = "Year")

p     <- ggplot(md, aes(x = as.numeric(as.character(Year)), y = value, group = variable, shape = variable)) + geom_point() + geom_line() + theme_tufte()
p     <- p + geom_rangeframe()+ xlab("") + ylab("Share %") + scale_shape_discrete(name = "", guide = "none") 
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5))
p     <- p + scale_y_continuous(limits = c(0,0.5), labels = percent) 
p.3A.primary.ties <- p


# Figure 3B - Number of ties with bankers and unionists ----
d.ties <- data.frame(Year = period.names, "Unionists" = number.of.union.send, "Bankers" = number.of.bank.send, check.names = FALSE)
t.n.type.ties <- d.ties

md    <- melt(d.ties, id.vars = "Year")
p     <- ggplot(md, aes(x = as.numeric(as.character(Year)), y = value, group = variable, shape = variable, color = variable)) + geom_point() + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "", start = 0.6, end = 0) + geom_rangeframe()+ xlab("") + ylab("Ties") + scale_shape_discrete(name = "") 
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5))
p.3B.n.type.ties <- p

# Figure 3C - Share of ties between firms with bankers and unionists ----
d.ties <- data.frame(Year = period.names, "Unionists" = andel.af.simple.directed.edges.der.er.union, "Bankers" = andel.af.simple.directed.edges.der.er.bank, check.names = FALSE)

t.type.ties <- d.ties
md    <- melt(d.ties, id.vars = "Year")
p     <- ggplot(md, aes(x = as.numeric(as.character(Year)), y = value, group = variable, shape = variable, color = variable)) + geom_point() + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "", start = 0.6, end = 0) + geom_rangeframe()+ xlab("") + ylab("Share %") + scale_shape_discrete(name = "") 
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5))
p     <- p + scale_y_continuous( labels = percent) 
p.3C.type.ties <- p

# Figure 3D - Number of firms dependent on unionist and bankers ----
d.com <- data.frame("Year" = period.names, "Dependent on unionists" = (com.size.uden.union - com.sizes) * -1, "Dependent on bankers" = (com.size.uden.bank - com.sizes) * -1, check.names = FALSE)

md    <- melt(d.com, id.vars = "Year")
p     <- ggplot(md, aes(x = as.numeric(as.character(Year)), y = value, group = variable, shape = variable, color = variable)) + geom_point() + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "", start = 0.6, end = 0) + geom_rangeframe()+ xlab("") + ylab("Number of disconnected firms") + scale_shape_discrete(name = "") 
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5))
p.3D.com.without <- p

# Figure 4 - Number of chemical companies on major bank boards or in VCI vorstand
md              <- melt(t.vci.bank.members)
md              <- md[md$variable == "Connected.to",]

p     <- ggplot(na.omit(md), aes(x = as.numeric(year), y = value, group = Type, shape = Type, color = Type)) + geom_point(size = 1.5) + geom_line() + theme_tufte()
p     <- p + scale_color_grey(name = "", start = 0.6, end = 0) + geom_rangeframe(color = "black") + xlab("") + ylab("Chemical firms") + scale_shape_discrete(name = "")
p     <- p + scale_x_continuous(breaks = seq(from = 1940, to = 2020, by = 10), minor_breaks = seq(from = 1940, to = 2020, by = 5)) + scale_y_continuous(limits = c(0,NA))
p.4.vci.bank.members <- p

# Appendix 1 ----

stills <- seq(1950, 2015, by = 1)
stills.set <- 1950:2015 %in% stills

l.lay <- list()

lay.past <- function(g1, g2, lay = NULL){
  if(is.null(lay)) lay <- layout_with_fr(g1)
  
  m2 <- match(V(g2)$name, V(g1)$name)
  
  lay.2 <- layout_with_fr(g2)
  lay.2[,] <- lay[m2,]
  lay.2[is.na(lay.2)] <- mean(lay.2, na.rm = TRUE)
  layout_with_fr(g2, coords = lay.2)
}

l.lay[[1]] <- layout_with_fr(g.list[stills.set][[1]])
for(i in 2:length(stills)) l.lay[[i]] <- lay.past(g.list[stills.set][[i-1]], g.list[stills.set][[i]], lay = l.lay[[i-1]])

activity.stills <- list()
for(i in 1:length(stills)) activity.stills[[i]]  <- activity.plot.small(g.list[stills.set][[i]], layout = l.lay[[i]])  
for(i in 1:length(stills)) activity.stills[[i]] <- activity.stills[[i]] + ggtitle(stills[i]) + theme(title = element_text(size = 7)) + coord_fixed()

# Save ----
save.image(file = "saved/3_plot.Rda")


